/**
 * 
 */
package sim;

/**
 * Class to model a simulation event 
 * @author Tanmay Bhola
 *
 */
public class Event implements Comparable<Event>{
	
	// Fields
	private int timing;
	private Wire wire;
	private LogicLevel level;
	
	/**
	 * Constructor for a simulation event
	 * @param timing The timing of the event in nanoseconds
	 * @param wire The Wire for which the logic level needs to be updated
	 * @param level The new logic level 
	 */
	public Event(int timing, Wire wire, LogicLevel level){
		this.timing = timing;
		this.wire = wire;
		this.level = level;		
	}
	
	/**
	 * The method to execute the event 
	 */
	public void fire(){
		wire.setLogicLevel(level);
	}

	/**
	 * @return the timing
	 */
	public int getTiming() {
		return timing;
	}

	/**
	 * @param timing the timing to set
	 */
	public void setTiming(int timing) {
		this.timing = timing;
	}

	/**
	 * @return the wire
	 */
	public Wire getWire() {
		return wire;
	}

	/**
	 * @param wire the wire to set
	 */
	public void setWire(Wire wire) {
		this.wire = wire;
	}

	/**
	 * @return the level
	 */
	public LogicLevel getLevel() {
		return level;
	}

	/**
	 * @param level the level to set
	 */
	public void setLevel(LogicLevel level) {
		this.level = level;
	}

	// Sorting based on event timing
	@Override
	public int compareTo(Event arg0) {
		if(this.getTiming()<arg0.getTiming()){
			return -1;
		}else if(this.getTiming() == arg0.getTiming()){
			return 0;
		}
		return 1;
	}
}
