/**
 * 
 */
package sim;

import java.util.Observable;
import java.util.Observer;

/**
 * Class to represent a port on a component
 * 
 * @author Tanmay
 *
 */
public class Port implements Observer {

	// Fields
	private Component component;
	private PortDirection direction;
	private LogicLevel logicLevel;
	private Wire connectedWire;
	private boolean isWireConnected;
	
	/**
	 * Constructor for port objects 
	 * @param component The component which contains this port
	 * @param direction The direction of the port
	 */
	public Port(Component component, PortDirection direction){
		
		this.component= component;
		this.direction = direction;
		this.logicLevel = LogicLevel.LOW;
		this.connectedWire = null;
		this.isWireConnected = false;
	}
	
	/**
	 * Method to assign a wire to a port
	 * @param w The Wire to assign
	 */
	public void assignWire(Wire w){
		
		// If its an input port it needs to listen to updates from 
		// the Wire that is attached to it. 
		if(direction == PortDirection.INPUT){
			// Check if the wire is being replaced by a new one
			if(isWireConnected && connectedWire != null){
				connectedWire.deleteObserver(this);
			}
			w.addObserver(this);
			
		}
		
		connectedWire = w;
		isWireConnected = true;
	}
	
	@Override
	public void update(Observable arg0, Object arg1) {
		if(arg0 instanceof Wire){
			setLogicLevel((LogicLevel) arg1);
			// Forwards the change in value to the component
			component.operation();
		}
	}
	
	
	/**
	 * @return the value
	 */
	public LogicLevel getLogicLevel() {
		return logicLevel;
	}

	/**
	 * @param value the value to set
	 */
	public void setLogicLevel(LogicLevel logicLevel) {
		this.logicLevel = logicLevel;
	}

	/**
	 * @return the component
	 */
	public Component getComponent() {
		return component;
	}

	/**
	 * @param component the component to set
	 */
	public void setComponent(Component component) {
		this.component = component;
	}

	/**
	 * @return the direction
	 */
	public PortDirection getDirection() {
		return direction;
	}

	/**
	 * @return the connectedWire
	 */
	public Wire getConnectedWire() {
		return connectedWire;
	}

	/**
	 * @return the isWireConnected
	 */
	public boolean isWireConnected() {
		return isWireConnected;
	}

	/**
	 * @param isWireConnected the isWireConnected to set
	 */
	public void setWireConnected(boolean isWireConnected) {
		this.isWireConnected = isWireConnected;
	}


}
