/**
 * 
 */
package sim;

import java.util.PriorityQueue;
import java.util.Queue;

/**
 * The simulator for running a Digital Logic Simulation
 * 
 * @author Tanmay
 *
 */
public class Simulator {
	
	// Fields
	private static Simulator uniqueSimulator;
	private int currentTime;
	private Queue<Event> events;
	
	// The private constructor
	private Simulator(){
		events = new PriorityQueue<Event>(); 
		currentTime = 0;
	}
	
	/**
	 * Method to get an instance of the simulator
	 * @return The simulator for running the simulation
	 */
	public static Simulator getSimulator(){
		if(uniqueSimulator == null){
			uniqueSimulator =  new Simulator();
		}
		
		return uniqueSimulator;
	}
	
	/**
	 * Method to run the simulation
	 */
	public void runSimulation(){
		// Keep executing events till the PQ is empty
		while(events.peek()!=null){
			Event e = events.poll();
			// Seek the simulation 
			setCurrentTime(e.getTiming());
			// Execute the event
			e.fire();
		}
	}
	
	/**
	 * Methos to add a new event to the queue
	 * @param wire The wire to update
	 * @param logicLevel The new logic level
	 * @param propagationDelay The delta for execution
	 */
	public void addEvent(Wire wire, LogicLevel logicLevel, int propagationDelay){
		events.add(new Event(currentTime+propagationDelay, wire, logicLevel));
	}

	/**
	 * @return the currentTime
	 */
	public int getCurrentTime() {
		return currentTime;
	}

	/**
	 * @param currentTime the currentTime to set
	 */
	private void setCurrentTime(int currentTime) {
		this.currentTime = currentTime;
	}
	
}
